@@ -37,6 +37,7 @@
 #include "rt-select.h"
 #include "rt-util.h"
 #include "rt-wumpus.h"
+#include "charsubst.h"
 #include "INTERN.h"
 #include "art.h"
 
@@ -78,7 +79,7 @@ art_init()
     ;
 }
 
-#ifdef MIME_SUPPORT
+#ifdef MIMESHOW
 #define VERY_LONG_STRING        200
 int
 display_mime()
@@ -96,8 +97,8 @@ display_mime()
 #endif
 	mode = 'p';
 	getcmd(buf);
-	mode = oldmode;
 	setdef(buf,"y");
+	mode = oldmode;
 #ifdef VERIFY
 	printcmd();
 #endif
@@ -137,25 +138,31 @@ do_article()
     bool notesfiles = FALSE;		/* might there be notesfiles junk? */
     char oldmode = mode;
     char *ctime();
-#ifdef MIME_SUPPORT
+#ifdef MIMESHOW
     bool tried_display_mime = FALSE;
 #endif
 #ifdef INNERSEARCH
-    register int outputok;
+    register int outputok = TRUE;
 #endif
 
-#ifdef MIME_SUPPORT
+#if defined(MIMESHOW) || defined(MIMESTORE)
     mime_article = FALSE;
 #endif
 
     if (fstat(fileno(artfp),&filestat)) /* get article file stats */
 	return DA_CLEAN;
-    if ((filestat.st_mode & S_IFMT) != S_IFREG)
+    if (!S_ISREG(filestat.st_mode))
 	return DA_NORM;
     artsize = filestat.st_size;		/* from that get article size */
+#ifdef CHARSUBST
+    sprintf(prompt_buf,
+	"%%sEnd of article %ld (of %ld) %%s-- what next? [%%s]",
+	(long)art,(long)lastart);	/* format prompt string */
+#else
     sprintf(prompt_buf,
 	"%%sEnd of article %ld (of %ld) -- what next? [%%s]",
 	(long)art,(long)lastart);	/* format prompt string */
+#endif
     prompt = prompt_buf;
     int_count = 0;		/* interrupt count is 0 */
     if ((firstpage = (topline < 0)) != 0) {
@@ -285,7 +292,7 @@ do_article()
 		}
 		else if (in_header == EXPIR_LINE) {
 		    if (!(htype[EXPIR_LINE].ht_flags & HT_HIDE))
-			hide_this_line = (strlen(art_buf) < 10);
+			hide_this_line = ((int)strlen(art_buf) < 10);
 		}
 		else if (in_header == FROM_LINE) {
 		    if (do_hiding && (s = extract_name(art_buf+6)) != Nullch)
@@ -299,7 +306,7 @@ do_article()
 				 localtime(&curr_artp->date));
 		}
 #endif
-#ifdef MIME_SUPPORT
+#if defined(MIMESHOW) || defined(MIMESTORE)
 		else if (!isspace(*art_buf)) {
 		    int nontext _((char *));
 		    int nonprint _((char *));
@@ -337,7 +344,7 @@ do_article()
 		linenum += tree_puts(art_buf,linenum+topline,0)-1;
 	    }
 	    else {			/* just a normal line */
-#ifdef MIME_SUPPORT
+#ifdef MIMESHOW
 		if (mime_article && do_hiding && !tried_display_mime) {
 		    if (display_mime() == 0)
 			return DA_NORM;
@@ -345,6 +352,12 @@ do_article()
 			tried_display_mime = TRUE;
 		}
 #endif
+#ifdef CLEAREOL
+#ifdef INNERSEARCH
+		if (outputok)
+#endif
+		if (can_home_clear) erase_eol();	
+#endif /* CLEAREOL */
 		if (highlight==artline) {	/* this line to be highlit? */
 		    if (marking == STANDOUT) {
 #ifdef NOFIREWORKS
@@ -367,12 +380,6 @@ do_article()
 		outputok = !hide_everything;
 					/* get it into register, hopefully */
 #endif
-#ifdef CLEAREOL
-#ifdef INNERSEARCH
-		if (outputok)
-#endif
-		maybe_eol();	
-#endif /* CLEAREOL */
 #ifdef CUSTOMLINES
 		if (pagestop && bufptr == art_buf && 
 		  execute(&page_compex,art_buf) )
@@ -380,7 +387,7 @@ do_article()
 #endif
 		for (outpos = 0; outpos < COLS; ) {
 				    /* while line has room */
-		    if (*(unsigned char *)bufptr >= ' ') { /* normal char? */
+		    if (*(unsigned char*)bufptr >= ' ') { /* normal char? */
 #ifdef ULSMARTS
 			if (*bufptr == '_') {
 			    if (bufptr[1] == '\b') {
@@ -407,9 +414,9 @@ do_article()
 				under_lining = 0;
 				un_underline();
 				if (UG) {
+				    outpos++;
 				    if (*bufptr == ' ')
 					goto skip_put;
-				    outpos++;
 				}
 			    }
 			}
@@ -425,12 +432,20 @@ do_article()
 				    putchar(*bufptr+13);
 				else
 				    putchar(*bufptr-13);
+				outpos++;
 			    }
 			    else
 #endif
+			    {
+#ifdef CHARSUBST
+				outpos += putsubstchar(*bufptr);
+#else
 				putchar(*bufptr);
+				outpos++;
+#endif
+			    }
 			}
-			if (*UC && ((highlight==artline && marking == 1)
+			if (*UC && ((highlight==artline && marking == STANDOUT)
 #ifdef ULSMARTS
 			    || under_lining
 #endif
@@ -440,7 +455,6 @@ do_article()
 			}
 		    skip_put:
 			bufptr++;
-			outpos++;
 		    }
 		    else if (*bufptr == '\n' || !*bufptr) {
 						    /* newline? */
@@ -492,11 +506,14 @@ do_article()
 			if (outputok)
 #endif
 			{
-			    if (dont_filter_control)
+			    if (dont_filter_control) {
 				putchar(*bufptr);
+				outpos++;
+			    }
 			    else {
 				putchar('^');
-				if (highlight == artline && *UC && marking == 1) {
+				if (highlight == artline && *UC
+				 && marking == STANDOUT) {
 				    backspace();
 				    underchar();
 				    putchar(*bufptr+64);
@@ -505,10 +522,10 @@ do_article()
 				}
 				else
 				    putchar(*bufptr+64);
+				outpos += 2;
 			    }
 			}
 			bufptr++;
-			outpos += 2;
 		    }
 		    
 		} /* end of column loop */
@@ -535,6 +552,9 @@ do_article()
 		    else
 			un_underline();
 		    highlight = -1;	/* no more we are */
+		    /* in case terminal highlighted rest of line earlier */
+		    /* when we did an eol with highlight turned on: */
+		    if (can_home_clear) erase_eol();
 		}
 		artline++;	/* count the line just printed */
 		if (artline - LINES + 1 > topline)
@@ -582,7 +602,12 @@ do_article()
 reask_pager:		    
 	unflush_output();	/* disable any ^O in effect */
 	standout();		/* enter standout mode */
+#ifdef CHARSUBST
+	printf("%s--MORE--(%ld%%)",current_charsubst(),
+	       (long)(artpos*100/artsize));
+#else
 	printf("--MORE--(%ld%%)",(long)(artpos*100/artsize));
+#endif
 	un_standout();	/* leave standout mode */
 #ifdef CLEAREOL
  	maybe_eol();
@@ -614,14 +639,12 @@ reask_pager:
 	    }
 	}
 	carriage_return();
-#ifndef CLEAREOL
-	erase_eol();	/* and erase the prompt */
-#else
+#ifdef CLEAREOL
 	if (erase_screen && can_home_clear)	
 	    clear_rest();
 	else
+#endif
 	    erase_eol();	/* and erase the prompt */
-#endif /* CLEAREOL */
 	carriage_return();	/* Resets kernel's tab column counter to 0 */
 	fflush(stdout);
 
@@ -653,31 +676,31 @@ page_switch()
     register char *s;
     
     switch (*buf) {
-    case 'd':
-    case Ctl('d'):	/* half page */
-	special = TRUE;
-	slines = LINES / 2 + 1;
-	if (marking && *blinebeg != '\f'
-#ifdef CUSTOMLINES
-	  && (!pagestop || blinebeg != art_buf ||
-	      !execute(&page_compex,blinebeg))
-#endif
-	  ) {
-	    up_line();
-	    highlight = --artline;
-	    restart = blinebeg;
-	    artpos = alinebeg;
-	}
-	return PS_NORM;
     case '!':			/* shell escape */
 	escapade();
 	return PS_ASK;
 #ifdef INNERSEARCH
-    case Ctl('i'):
+    case Ctl('i'): {
+	ART_LINE i = artline;
+	ART_POS pos;
 	gline = 3;
-	sprintf(cmd_buf,"^[^%c]",*blinebeg);
+	s = blinebeg;
+	while (*s == '\n' && i >= topline) {
+	    pos = vrdary(--i);
+	    if (pos < 0)
+		pos = -pos;
+	    if (pos < htype[PAST_HEADER].ht_minpos)
+		break;
+	    fseek(artfp,(long)pos,0);
+	    if (fgets(s = buf+1, sizeof buf - 1, artfp) == Nullch) {
+		s = blinebeg;
+		break;
+	    }
+	}
+	sprintf(cmd_buf,"^[^%c\n]",*s);
 	compile(&gcompex,cmd_buf,TRUE,TRUE);
 	goto caseG;
+    }
     case Ctl('g'):
 	gline = 3;
 	compile(&gcompex,"^Subject:",TRUE,TRUE);
@@ -699,7 +722,6 @@ page_switch()
 	/* FALL THROUGH */
     caseG:
     case 'G': {
-	/* ART_LINE lines_to_skip = 0; */
 	ART_POS start_where;
 
 	if (gline < 0 || gline > LINES-2)
@@ -722,7 +744,6 @@ page_switch()
 	fseek(artfp,(long)start_where,0);
 	innersearch = 0; /* assume not found */
 	while (fgets(buf, sizeof buf, artfp) != Nullch) {
-	    /* lines_to_skip++; 		NOT USED NOW */
 #ifdef DEBUG
 	    if (debug & DEB_INNERSRCH)
 		printf("Test %s",buf) FLUSH;
@@ -771,7 +792,7 @@ page_switch()
 	notincl("g");
 	return PS_ASK;
 #endif
-    case '\n':		/* one line */
+    case '\n':		/* one line down */
 	special = TRUE;
 	slines = 2;
 	return PS_NORM;
@@ -782,6 +803,7 @@ page_switch()
 #endif
     case 'l':
     case '\f':		/* refresh screen */
+    refresh_screen:
 #ifdef DEBUG
 	if (debug & DEB_INNERSRCH) {
 	    printf("Topline = %d",topline) FLUSH;
@@ -796,23 +818,55 @@ page_switch()
 	    artline = 0;
 	firstpage = (topline < 0);
 	return PS_NORM;
+    case 'B':		/* one line up */
+	if (topline < 0)
+	    break;
+	if (*IL && *HO) {
+	    ART_POS pos;
+	    home_cursor();
+	    insert_line();
+	    carriage_return();	/* Resets kernel's tab column counter to 0 */
+	    pos = vrdary(topline-1);
+	    if (pos < 0)
+		pos = -pos;
+	    if (pos >= htype[PAST_HEADER].ht_minpos) {
+		fseek(artfp,(long)pos,0);
+		if (fgets(s = buf, sizeof buf, artfp) != Nullch) {
+		    fputs(buf, stdout) FLUSH;
+		    topline--;
+		    artpos = vrdary(--artline);
+		    if (artpos < 0)
+			artpos = -artpos;
+		    fseek(artfp,artpos,0);
+		    goto_line(1, LINES-1);
+		    erase_eol();
+		    carriage_return();
+		    return PS_ASK;
+		}
+	    }
+	}
+	/* FALL THROUGH */
     case 'b':
     case '\b':
     case Ctl('b'): {	/* back up a page */
 	ART_LINE target;
 
-#ifndef CLEAREOL
-	clear();
-#else
-	if (can_home_clear)	/* if we can home do it */
+#ifdef CLEAREOL
+	if (can_home_clear)	/* if we can let home do it */
 	    home_cursor();
 	else
+#endif
 	    clear();
-
-#endif /* CLEAREOL */
 	carriage_return();	/* Resets kernel's tab column counter to 0 */
+
 	do_fseek = TRUE;	/* reposition article file */
-	target = topline - (LINES - 2);
+	if (*buf == 'B')
+	    target = topline - 1;
+	else {
+	    target = topline - (LINES - 2);
+	    if (marking && (marking_areas & BACKPAGE_MARKING))
+		highlight = topline;
+	}
 	artline = topline;
 	if (artline >= 0) do {
 	    artline--;
@@ -840,6 +894,12 @@ page_switch()
 	if (!finish_dblchar())
 	    return PS_ASK;
 	switch (buf[1] & 0177) {
+#ifdef CHARSUBST
+	case 'C':
+	    if (!*(++charsubst))
+		charsubst = charsets;
+	    goto refresh_screen;
+#endif
 	default:
 	    goto leave_pager;
 	}
@@ -893,7 +953,7 @@ page_switch()
 #endif
     case Ctl('x'):
     case 'z':	case 'Z':
-    case '^':
+    case '^':   case Ctl('^'):
 leave_pager:
 #ifdef ROTATION
 	rotate = FALSE;
@@ -904,7 +964,11 @@ leave_pager:
 	  index("wWsSe:!&|/?123456789.",*buf) != Nullch) {
 	    setdfltcmd();
 	    standout();		/* enter standout mode */
+#ifdef CHARSUBST
+	    printf(prompt,mailcall,current_charsubst(),dfltcmd);
+#else
 	    printf(prompt,mailcall,dfltcmd);
+#endif
 			    /* print prompt, whatever it is */
 	    un_standout();	/* leave standout mode */
 	    putchar(' ');
@@ -916,51 +980,51 @@ leave_pager:
 	rotate = TRUE;
 	/* FALL THROUGH */
 #endif
+    case 'd':		/* half page */
+    case Ctl('d'):
+	special = TRUE;
+	slines = LINES / 2 + 1;
+	/* no divide-by-zero, thank you */
+	if (LINES > 2 && (LINES & 1) && artline % (LINES-2) >= LINES/2 - 1)
+	    slines++;
+	goto go_forward;
     case 'y':
-    case Ctl('v'):
-					/* Leaving it undocumented in case */
+    case Ctl('v'):			/* Leaving it undocumented in case */
 					/* I want to steal the key--LAW */
     case ' ':	/* continue current article */
 	if (erase_screen) {	/* -e? */
-#ifndef CLEAREOL
-	    clear();		/* clear screen */
-#else
-	    if (can_home_clear)	/* if we can home do it */
+#ifdef CLEAREOL
+	    if (can_home_clear)	/* if we can let home do it */
 		home_cursor();
 	    else
-		clear();	/* else clear screen */
-
-#endif /* CLEAREOL */
+#endif
+		clear();	/* clear screen */
 	    carriage_return();	/* Resets kernel's tab column counter to 0 */
 	    fflush(stdout);
-
-	    if (*blinebeg != '\f'
-#ifdef CUSTOMLINES
-	      && (!pagestop || blinebeg != art_buf ||
-	          !execute(&page_compex,blinebeg))
-#endif
-	      ) {
-		restart = blinebeg;
-		artline--;	 /* restart this line */
-		artpos = alinebeg;
-		if (marking)	/* and mark repeated line */
-		    highlight = artline;
-	    }
-	    topline = artline;
-			/* and remember top line of screen */
-			/*  (line # within article file) */
 	}
-	else if (marking && *blinebeg != '\f'
+	else {
+	    special = TRUE;
+	    slines = LINES;
+	}
+    go_forward:
+	if (*blinebeg != '\f'
 #ifdef CUSTOMLINES
 	  && (!pagestop || blinebeg != art_buf ||
 	      !execute(&page_compex,blinebeg))
 #endif
 	  ) {
-				/* are we marking repeats? */
-	    up_line();		/* go up one line */
-	    highlight = --artline;/* and get ready to highlight */
-	    restart = blinebeg;	/*   the old line */
-	    artpos = alinebeg;
+	    if (!special
+	     || (marking && (*buf!='d' || (marking_areas&HALFPAGE_MARKING)))) {
+		restart = blinebeg;
+		artline--;	 /* restart this line */
+		artpos = alinebeg;
+		if (special)
+		    up_line();
+		if (marking)
+		    highlight = artline;
+	    }
+	    else
+		slines--;
 	}
 	return PS_NORM;
     case 'q':	/* quit this article? */
@@ -971,6 +1035,7 @@ leave_pager:
 	settle_down();
 	return PS_ASK;
     }
+    return PS_ASK;
 }
 
 #ifdef INNERSEARCH
@@ -1012,7 +1077,7 @@ innermore()
 }
 #endif
 
-#ifdef MIME_SUPPORT
+#if defined(MIMESHOW) || defined(MIMESTORE)
 int
 nontext(s)
 char *s;
