@@ -66,7 +66,6 @@ int get_cmd;				/*   be set to FALSE!!! */
     bool doread;			/* search read articles? */
     bool foldcase = TRUE;		/* fold upper and lower case? */
     int ignorethru = 0;			/* should we ignore the thru line? */
-
     ART_NUM srchfirst;
 
     int_count = 0;
@@ -166,10 +165,14 @@ int get_cmd;				/*   be set to FALSE!!! */
 	    if (cmdchr != 'k')
 		saltaway = TRUE;
 	    normal_return = SRCH_DONE;
-	    if (cmdchr == '+')
-		cmdlst = savestr("I:++");
-	    else if (cmdchr == '.')
-		cmdlst = savestr("I:.");
+	    if (cmdchr == '+') {
+		cmdlst = savestr("+");
+		ignorethru = 1;
+	    }
+	    else if (cmdchr == '.') {
+		cmdlst = savestr(".");
+		ignorethru = 1;
+	    }
 	    else {
 		if (cmdchr == ',')
 		    cmdlst = savestr(",");
@@ -230,6 +233,8 @@ int get_cmd;				/*   be set to FALSE!!! */
 	*s++ = '/';
 	if (doread)
 	    *s++ = 'r';
+	if (ignorethru)
+	    *s++ = (ignorethru == 1 ? 'I' : 'N');
 	if (howmuch != ARTSCOPE_SUBJECT) {
 	    *s++ = scopestr[howmuch];
 	    if (howmuch == ARTSCOPE_ONEHDR) {
