@@ -71,6 +71,8 @@ rc_to_bits()
     s = mybuf;				/* initialize the for loop below */
     if (strnEQ(s,"1-",2)) {		/* can we save some time here? */
 	firstart = atol(s+2)+1;		/* process first range thusly */
+	if (firstart < absfirst)
+	    firstart = absfirst;
 	s=index(s,',') + 1;
 	for (i = absfirst, ap = article_ptr(i); i < firstart; i++, ap++)
 	    ap->flags |= AF_READ;
@@ -250,7 +252,7 @@ setfoundbits()
 {
     register ART_NUM first = lastart+1;
     register DIR *dirp;
-    register struct direct *dp;
+    register Direntry_t *dp;
     long an;
     char ch;
 
@@ -262,7 +264,7 @@ setfoundbits()
     found_bits = safemalloc((MEM_SIZE)an);
     bzero(found_bits, an);
 
-    while ((dp = readdir(dirp)) != Null(struct direct *)) {
+    while ((dp = readdir(dirp)) != Null(Direntry_t*)) {
 	if (sscanf(dp->d_name, "%ld%c", &an, &ch) == 1) {
 	    if (an <= lastart && an >= found_min) {
 		if (an < first)
