@@ -32,6 +32,8 @@
 #include "hash.h"
 #include "rthread.h"
 #include "rt-ov.h"
+#include "rt-page.h"
+#include "rt-process.h"
 #include "rt-select.h"
 #include "rt-util.h"
 #include "util.h"
@@ -43,6 +45,7 @@
 #endif
 
 HASHTABLE *subj_hash = 0;
+HASHTABLE *shortsubj_hash = 0;
 
 int subject_cmp _((char *,int,HASHDATUM));
 
@@ -158,6 +161,10 @@ close_cache()
 	hashdestroy(subj_hash);
 	subj_hash = 0;
     }
+    if (shortsubj_hash) {
+	hashdestroy(shortsubj_hash);
+	shortsubj_hash = 0;
+    }
     /* Free all the subjects. */
     for (sp = first_subject; sp; sp = next) {
 	next = sp->next;
@@ -204,18 +211,59 @@ register ARTICLE *ap;
     ap->flags |= AF_CACHED;
 
     if (!(ap->flags & AF_READ) ^ sel_rereading) {
-	if (selected_only) {
-	    if (ap->subj->flags & sel_mask) {
-		ap->flags |= sel_mask;
-		selected_count++;
-	    }
-	} else if (ap->subj->flags & SF_WASSELECTED)
+	if (ap->subj->flags & SF_WASSELECTED)
 	    select_article(ap, 0);
-	else {
+	else
 	    ap->subj->flags |= SF_VISIT;
-	    if (sel_mode == SM_THREAD)
-		ap->subj->thread->subj->flags |= SF_VISIT;
+    }
+
+    if (join_subject_len != 0)
+	check_for_near_subj(ap);
+}
+
+void
+check_for_near_subj(ap)
+ARTICLE *ap;
+{
+    register SUBJECT *sp;
+    if (!shortsubj_hash) {
+	shortsubj_hash = hashcreate(201, subject_cmp);	/*TODO: pick a better size */
+	sp = first_subject;
+    }
+    else {
+	sp = ap->subj;
+	if (sp->next)
+	    sp = 0;
+    }
+    while (sp) {
+	if (strlen(sp->str+4) >= join_subject_len && sp->thread) {
+	    SUBJECT *sp2;
+	    HASHDATUM data;
+	    data = hashfetch(shortsubj_hash, sp->str+4, join_subject_len);
+	    if (!(sp2 = (SUBJECT*)data.dat_ptr)) {
+		data.dat_ptr = (char*)sp;
+		hashstorelast(data);
+	    }
+	    else if (sp->thread != sp2->thread) {
+		merge_threads(sp2, sp);
+	    }
 	}
+	sp = sp->next;
+    }
+}
+
+void
+change_join_subject_len(len)
+int len;
+{
+    if (join_subject_len != len) {
+	if (shortsubj_hash) {
+	    hashdestroy(shortsubj_hash);
+	    shortsubj_hash = 0;
+	}
+	join_subject_len = len;
+	if (len && first_subject && first_subject->articles)
+	    check_for_near_subj(first_subject->articles);
     }
 }
 
@@ -223,7 +271,7 @@ void
 check_poster(ap)
 register ARTICLE *ap;
 {
-    if (auto_select_postings && !(ap->flags & AF_MISSING)) {
+    if (auto_select_postings && !(ap->flags & AF_MISSING) && ap->from) {
 	if (ap->flags & AF_FROMTRUNCED) {
 	    strcpy(cmd_buf,realname);
 	    if (strEQ(ap->from,compress_name(cmd_buf,16))) {
@@ -253,9 +301,22 @@ register ARTICLE *ap;
 	    } else
 		h = u = s;
 	    if (strEQ(u,loginName)) {
-		if (instr(h,phostname,FALSE))
-		    select_subthread(ap,AF_AUTOSELECT);
-		else {
+		if (instr(h,phostname,FALSE)) {
+		    switch (auto_select_postings) {
+		    case '.':
+			select_subthread(ap,AF_AUTOSELECT);
+			break;
+		    case '+':
+			select_arts_thread(ap,AF_AUTOSELECT);
+			break;
+		    case 'p':
+			if (ap->parent)
+			    select_subthread(ap->parent,AF_AUTOSELECT);
+			else
+			    select_subthread(ap,AF_AUTOSELECT);
+			break;
+		    }
+		} else {
 #ifdef REPLYTO_POSTER_CHECKING
 		    char *reply_buf = fetchlines(article_num(ap),REPLY_LINE);
 		    if (instr(reply_buf,loginName,TRUE))
@@ -414,6 +475,8 @@ register int size;
 	else
 	    f++;
     }
+    while (size > 4 && t[-1] == ' ')
+	t--, size--;
     *t = '\0';
 
     if (ap->subj) {
@@ -436,7 +499,7 @@ register int size;
 	    last_subject = sp;
 	    sp->str = newsubj;
 	    sp->thread_link = sp;
-	    sp->flags = SF_THREAD;
+	    sp->flags = 0;
 
 	    data.dat_ptr = (char*)sp;
 	    hashstorelast(data);
@@ -798,6 +861,8 @@ ART_NUM last;
 	}
     }
     if (success && last_cached < last) {
+	if (ov_opened)
+	    ov_data(last_cached+1, last, FALSE);
 	success = art_data(last_cached+1, last, FALSE, all_arts);
 	cached_all_in_range = (all_arts && success);
     }
