@@ -35,7 +35,6 @@ typedef struct rt_subj {
 #define SF_OLDSEL	0x0008
 #define SF_INCLUDED	0x0010
 
-#define SF_THREAD	0x0100
 #define SF_VISIT	0x0200
 #define SF_WASSELECTED  0x0400
 #define SF_SUBJTRUNCED	0x1000
@@ -130,6 +129,8 @@ void	build_cache _((void));
 void	grow_cache _((ART_NUM));
 void	close_cache _((void));
 void	cache_article _((ARTICLE*));
+void	check_for_near_subj _((ARTICLE*));
+void	change_join_subject_len _((int));
 void	check_poster _((ARTICLE*));
 void	uncache_article _((ARTICLE*,bool_int));
 char	*fetchcache _((ART_NUM,int,bool_int));
