@@ -90,6 +90,7 @@ int status;
     termlib_reset();
     if (bizarre)
 	resetty();
+    xmouse_off();	/* turn off mouse tracking (if on) */
     if (lockname && *lockname)
  	UNLINK(lockname);
 #ifdef USE_NNTP
@@ -222,6 +223,9 @@ stop_catcher(signo)
 int signo;
 {
     if (!waiting) {
+	bool mouse_flag;	/* if TRUE, mouse tracking was on */
+	mouse_flag = xmouse_status;
+	xmouse_off();
 	checkpoint_rc();	/* good chance of crash while stopped */
 	if (clear_on_stop) {
 	    clear();
@@ -252,6 +256,8 @@ int signo;
 		errno = 0;			/* needed for getcmd */
 	    }
     	}
+	if (mouse_flag)
+	    xmouse_on();
     }
     sigset(signo,stop_catcher);	/* unenable the stop */
 }
