@@ -48,6 +48,7 @@ CR	Display one more line.\n\
     (cmd = print_lines("\
 b	Back up one page.\n\
 ^L,X	Refresh the screen (X=rot13).\n\
+_C      Switch characterset conversion.\n\
 ",NOMARKING)) ||
     (cmd = print_lines("\
 t	Display the entire article tree and all its subjects.\n\
@@ -153,6 +154,7 @@ range{,range}:command{:command}\n\
     (cmd = print_lines("\
 f,F	Submit a followup article (F = include this article).\n\
 r,R	Reply through net mail (R = include this article).\n\
+^F	Forward article through net mail.\n\
 e dir{|command}\n\
 	Extract to directory using /bin/sh, uudecode, unship, or command.\n\
 s ...	Save to file or pipe via sh.\n\
@@ -164,6 +166,7 @@ w,W	Like s and S but save without the header.\n\
 C	Cancel this article, if yours.\n\
 ^R,v	Restart article (v=verbose).\n\
 ^X	Restart article, rot13 mode.\n\
+_C      Switch characterset conversion.\n\
 c	Catch up (mark all articles as read).\n\
 b	Back up one page.\n\
 ",NOMARKING)) ||
@@ -396,7 +399,7 @@ Selection commands:\n\
 ",STANDOUT)) ||
     (cmd = print_lines("\n\
 a-z,0-9	Select/deselect the indicated item by its letter or number.  Many of\n\
-	the alpha letters are ommitted for the following commands.\n\
+	the alpha letters are omitted for the following commands.\n\
 SP	Perform the default command (usually > or Z).\n\
 CR	Start reading.  Selects the current item if nothing is selected.\n\
 Z,TAB	Start reading.  If nothing is selected, read all unread articles.\n\
@@ -462,6 +465,7 @@ P	Leave this group as-is and go on to the previous one.\n\
 &	View or set command line switches.\n\
 &&	View or set macro definitions.\n\
 !cmd	Escape to a subshell.\n\
+h, ?	This help message.\n\
 q	Quit the selector.\n\
 Q	Quit group and return to news group selection prompt for this group.\n\
 ",NOMARKING)) )
