@@ -53,6 +53,7 @@ char *argv[];
 {
     char *tcbuf;
     register bool foundany = FALSE;
+    char *s;
 #ifdef NOLINEBUF
     static char std_out_buf[BUFSIZ];	/* must be static or malloced */
 
@@ -191,6 +192,23 @@ char *argv[];
 # endif
 #endif
 
+/* xterm mouse initialization -- move somewhere different? */
+    /* values of XTERMMOUSE:  y/Y/1 -- if appropriate, use it. */
+    /*                        n/N/0 -- don't ever use it. */
+    /*                        a/A   -- always use it. */
+    s = getval("XTERMMOUSE","-");
+    if (*s == 'y' || *s == 'Y' || *s == '1') {
+	if ((s = getenv("TERM")) != Nullch && strnEQ(s,"xterm",5))
+	    use_xterm_mouse = TRUE;
+    }
+    else if (*s == 'a' || *s == 'A')
+	use_xterm_mouse = TRUE;
+    else if (*s != 'n' && *s != 'N' && *s != '0') {
+	s = argv[0];
+	/* if an 'x' is at the end, enable Xterm mouse tracking */
+	if (s[strlen(s)-1] == 'x')
+	    use_xterm_mouse = TRUE;
+    }
     return foundany;
 }
 
@@ -201,13 +219,14 @@ lock_check()
 {
     lockname = savestr(filexp(LOCKNAME));
     if (!checkflag) {
+	int processnum = 0;
 	tmpfp = fopen(lockname,"r");
 	if (tmpfp != Nullfp) {
-	    int processnum;
-    
-	    fgets(buf,LBUFLEN,tmpfp);
+	    if (fgets(buf,LBUFLEN,tmpfp))
+		processnum = atoi(buf);
 	    fclose(tmpfp);
-	    processnum = atoi(buf);
+	}
+	if (processnum) {
 #ifdef VERBOSE
 	    IF(verbose)
 		printf("You seem to have left a trn running, process %d.\n",
