@@ -145,7 +145,8 @@ char *tcbuf;
     /* host name that goes in postings (%H) */
 
     phostname = PHOSTNAME;
-    if (*phostname == '/') {
+    if (*phostname == '/' || *phostname == '~') {
+	phostname = filexp(phostname);
 	if ((tmpfp = fopen(phostname,"r")) == NULL) {
 	    printf("Warning: Couldn't open %s to determine hostname!\n",
 		   phostname); 
@@ -185,12 +186,9 @@ char *tcbuf;
 # endif /* HAS_UNAME */
 #endif /* HAS_GETHOSTNAME */
 	if (*buf) {
-	    char *cp = index(buf,'.');
-	    if (cp)
-		*cp = '\0';
-	    cp = index(phostname,'.');
-	    if (cp)
-		strcat(buf,cp);
+	    if (*phostname == '.' && phostname[1]) {
+		strcat(buf,phostname);
+	    }
 	    phostname = savestr(buf);
 	}
     }
@@ -353,7 +351,23 @@ char *stoppers;
 	    printf("skipinterp till %s at %s\n",stoppers?stoppers:"",pattern);
 #endif
 	if (*pattern == '%' && pattern[1]) {
+	switch_again:
 	    switch (*++pattern) {
+	    case '^':
+	    case '_':
+	    case '\\':
+	    case '\'':
+	    case '>':
+	    case ')':
+		goto switch_again;
+	    case ':':
+		pattern++;
+		while (*pattern
+		 && (*pattern=='.' || *pattern=='-' || isdigit(*pattern))) {
+		    pattern++;
+		}
+		pattern--;
+		goto switch_again;
 	    case '{':
 		for (pattern++; *pattern && *pattern != '}'; pattern++)
 		    if (*pattern == '\\')
@@ -435,6 +449,8 @@ char *stoppers;
     register int i;
     char scrbuf[512];
     char spfbuf[512];
+    static char *input_str = Nullch;
+    static int input_siz = 0;
     bool upper = FALSE;
     bool lastcomp = FALSE;
     bool re_quote = FALSE;
@@ -615,7 +631,10 @@ char *stoppers;
 		    gets(scrbuf);
 		    noecho();
 		    crmode();
-		    s = scrbuf;
+		    i = strlen(scrbuf);
+		    growstr(&input_str, &input_siz, i+1);
+		    safecpy(input_str, scrbuf, i+1);
+		    s = input_str;
 		    break;
 #endif
 		case '~':
@@ -755,18 +774,22 @@ char *stoppers;
 		    if (s == Nullch) 
 			s = getval("ORGANIZATION",orgname); 
 #endif
+		    s = filexp(s);
 #ifdef ORGFILE
 		    if (*s == '/') {
 			FILE *ofp = fopen(s,"r");
 
 			if (ofp) {
-			    fgets(scrbuf,sizeof scrbuf,ofp);
+			    if (fgets(scrbuf,sizeof scrbuf,ofp) == NULL)
+			    	*scrbuf = '\0';
 			    fclose(ofp);
 			    s = scrbuf+strlen(scrbuf)-1;
-			    if (*s == '\n')
+			    if (*scrbuf && *s == '\n')
 				*s = '\0';
 			    s = scrbuf;
 			}
+			else
+			    s = nullstr;
 		    }
 #endif
 		    break;
@@ -779,6 +802,9 @@ char *stoppers;
 		case 'P':
 		    s = spool;
 		    break;
+		case 'q':
+		    s = input_str;
+		    break;
 		case 'r':
 		    parseheader(art);
 		    if (htype[REFS_LINE].ht_minpos >= 0) {
@@ -1000,12 +1026,18 @@ char *stoppers;
 			abort_interp();
 		}
 		while (*s) {
-		    if ((re_quote && index(regexp_specials, *s))
-		     || (tick_quote && *s == '\'')) {
+		    if (re_quote && index(regexp_specials, *s)) {
 			if (--destsize <= 0)
 			    abort_interp();
 			*dest++ = '\\';
 		    }
+		    else if (tick_quote && *s == '\'') {
+			if ((destsize -= 3) <= 0)
+			    abort_interp();
+			*dest++ = '\'';
+			*dest++ = '\\';
+			*dest++ = '\'';
+		    }
 		    *dest++ = *s++;
 		}
 	    } else {
@@ -1073,7 +1105,7 @@ char *stoppers;
     }
     *dest = '\0';
     if (line_split != Nullch)
-	if (strlen(orig_dest) > 79)
+	if ((int)strlen(orig_dest) > 79)
 	    *line_split = '\n';
 getout:
     if (subj_buf != Nullch)	/* return any checked out storage */
