@@ -25,6 +25,7 @@
 #include "ngstuff.h"
 #include "rcstuff.h"
 #include "trn.h"
+#include "bits.h"
 #include "hash.h"
 #include "rthread.h"
 #include "rt-process.h"
@@ -85,6 +86,7 @@ FILE *kfp;
 int entering;
 {
     bool first_time = (entering && !killfirst);
+    bool append_nl = FALSE;
     ART_UNREAD selections = selected_count;
     ART_UNREAD unread = toread[ng];
     int thread_kill_cnt = 0;
@@ -99,6 +101,8 @@ int entering;
     while (fgets(buf,LBUFLEN,kfp) != Nullch) {
 	if (*(cp = buf + strlen(buf) - 1) == '\n')
 	    *cp = '\0';
+	else
+	    append_nl = TRUE;
 	if (strnEQ(buf,"THRU",4)) {
 	    killfirst = atol(buf+4)+1;
 	    if (killfirst < absfirst)
@@ -243,6 +247,8 @@ int entering;
 	fputs(".\n",stdout) FLUSH;
 	kill_mentioned = TRUE;
     }
+    if (append_nl)
+	kf_append("");
 #endif
     return 0;
 }
@@ -481,22 +487,25 @@ char *cmd;
 {
     strcpy(cmd_buf,filexp(getval("KILLLOCAL",killlocal)));
     if (makedir(cmd_buf,MD_FILE) >= 0) {
+	if (*cmd) {
 #ifdef VERBOSE
-	IF(verbose)
-	    printf("\nDepositing command in %s...",cmd_buf);
-	ELSE
+	    IF(verbose)
+		printf("\nDepositing command in %s...",cmd_buf);
+	    ELSE
 #endif
 #ifdef TERSE
-	    printf("\n--> %s...",cmd_buf);
+		printf("\n--> %s...",cmd_buf);
 #endif
-	fflush(stdout);
-	if (novice_delays)
-	    sleep(2);
+	    fflush(stdout);
+	    if (novice_delays)
+		sleep(2);
+	}
 	if ((tmpfp = fopen(cmd_buf,"a")) != Nullfp) {
 	    fseek(tmpfp,0L,2);		/* get to EOF for sure */
 	    fprintf(tmpfp,"%s\n",cmd);
 	    fclose(tmpfp);
-	    fputs("done\n",stdout) FLUSH;
+	    if (*cmd)
+		fputs("done\n",stdout) FLUSH;
 	}
 	else
 	    printf(cantopen,cmd_buf) FLUSH;
