@@ -41,6 +41,7 @@
 #include "rt-select.h"
 #include "rt-wumpus.h"
 #include "decode.h"
+#include "charsubst.h"
 #include "INTERN.h"
 #include "ng.h"
 #include "artstate.h"			/* somebody has to do it */
@@ -102,7 +103,11 @@ do_newsgroup(start_command)
 char *start_command;			/* command to fake up first */
 {
     char oldmode = mode;
+#ifdef CHARSUBST
+    char *whatnext = "%s%sWhat next? [%s]";
+#else
     char *whatnext = "%sWhat next? [%s]";
+#endif
 
 #ifdef ARTSEARCH
     srchahead = (scanon && !ThreadedGroup	/* did they say -S? */
@@ -129,7 +134,6 @@ char *start_command;			/* command to fake up first */
 
     /* FROM HERE ON, RETURN THRU CLEANUP OR WE ARE SCREWED */
 
-    in_ng = TRUE;			/* tell the world we are here */
     forcelast = TRUE;			/* if 0 unread, do not bomb out */
     recent_artp = curr_artp = Nullart;
     recent_art = curr_art = lastart+1;
@@ -178,10 +182,13 @@ char *start_command;			/* command to fake up first */
     do_fseek = FALSE;			/* start 1st article at top */
     for (; art<=lastart+1; ) {		/* for each article */
 	mode = 'a';
+#ifdef CHARSUBST
+	charsubst = charsets;
+#endif
 
 	/* do we need to "grow" the newsgroup? */
 
-	if (art > lastart || forcegrow) {
+	if ((art > lastart || forcegrow) && !keep_the_group_static) {
 	    ART_NUM oldlast = lastart;
 #ifdef USE_NNTP
 	    if (!nntp_group(ngname,ng)) {
@@ -290,8 +297,17 @@ char *start_command;			/* command to fake up first */
 					/*  (line # within article file) */
 	    }
 	    clear();			/* clear screen */
+#ifdef USE_NNTP
+	    if (art == 0 && artp && artp->msgid
+	     && !(artp->flags & AF_CACHED)) {
+		art = nntp_stat_id(artp->msgid);
+		if (art)
+		    artp = find_article(art);
+	    }
+#endif
 	    if (!artopen(art)) {	/* make sure article is found & open */
 		char tmpbuf[256];
+		ART_LINE linenum;
 		/* see if we have tree data for this article anyway */
 		init_tree();
 		sprintf(tmpbuf,"%s: article is not available.",ngname);
@@ -301,9 +317,9 @@ char *start_command;			/* command to fake up first */
 			sprintf(tmpbuf,"%s: article may show up in a moment.",
 				ngname);
 		}
-		tree_puts(tmpbuf,0,0);
-		vwtary((ART_LINE)0,(ART_POS)0);
-		finish_tree(1);
+		linenum = tree_puts(tmpbuf,0,0);
+		vwtary(artline,(ART_POS)0);
+		finish_tree(linenum);
 		prompt = whatnext;
 #ifdef ARTSEARCH
 		srchahead = 0;
@@ -342,7 +358,12 @@ reask_article:
 #endif /* CLEAREOL */
 	unflush_output();		/* disable any ^O in effect */
 	standout();			/* enter standout mode */
-	printf(prompt,mailcall,dfltcmd);/* print prompt, whatever it is */
+	/* print prompt, whatever it is */
+#ifdef CHARSUBST
+	printf(prompt,mailcall,current_charsubst(),dfltcmd);
+#else
+	printf(prompt,mailcall,dfltcmd);
+#endif
 	un_standout();			/* leave standout mode */
 	putchar(' ');
 	fflush(stdout);
@@ -432,31 +453,31 @@ art_switch()
 
     switch (*buf) {
     case '<':			/* goto previous subject/thread */
-	prev_subject();
+	visit_prev_thread();
 	return AS_NORM;
     case '>':			/* goto next subject/thread */
-	next_subject();
+	visit_next_thread();
 	return AS_NORM;
     case 'U': {			/* unread some articles */
 	char *u_prompt, *u_help_thread;
 
-	dfltcmd = "+";
 	if (!artp) {
 	    u_help_thread = nullstr;
 #ifdef VERBOSE
 	    IF(verbose)
-		u_prompt = "\nSet unread: +select or all? [+an] ";
+		u_prompt = "\nSet unread: +select or all?";
 	    ELSE
 #endif
 #ifdef TERSE
-		u_prompt = "\nSet unread? [+an] ";
+		u_prompt = "\nSet unread?";
 #endif
+	    dfltcmd = "+an";
 	}
 	else {
 #ifdef VERBOSE
 	    IF(verbose) {
 		u_prompt = "\n\
-Set unread: +select, thread, subthread, or all? [+tsan] ";
+Set unread: +select, thread, subthread, or all?";
 		u_help_thread = "\
 Type t or SP to mark this thread's articles as unread.\n\
 Type s to mark the current article and its descendants as unread.\n";
@@ -465,16 +486,16 @@ Type s to mark the current article and its descendants as unread.\n";
 #endif
 #ifdef TERSE
 	    {
-		u_prompt = "\nSet unread? [+tsan] ";
+		u_prompt = "\nSet unread?";
 		u_help_thread = "\
 t or SP to mark thread unread.\n\
 s to mark subthread unread.\n";
 	    }
 #endif
+	    dfltcmd = "+tsan";
 	}
       reask_unread:
-	in_char(u_prompt,'u');
-	setdef(buf,dfltcmd);
+	in_char(u_prompt,'u',dfltcmd);
 #ifdef VERIFY
 	printcmd();
 #endif
@@ -750,9 +771,11 @@ This is the last leaf in this tree.\n",stdout) FLUSH;
 	    return AS_NORM;
 	}
 #endif
-	else
+	else {
 	    inc_art(selected_only,FALSE);
-
+	    if (art > lastart)
+		top_article();
+	}
 #ifdef ARTSEARCH
 	srchahead = 0;
 #endif
@@ -1045,6 +1068,7 @@ run_the_selector:
 	return AS_ASK;
 #endif
     case 'l': case Ctl('l'):		/* refresh screen */
+    refresh_screen:
 	if (art <= lastart) {
 	    reread = TRUE;
 	    clear();
@@ -1054,13 +1078,14 @@ run_the_selector:
 		artline = 0;
 	}
 	return AS_NORM;
-    case Ctl('f'):
+    case Ctl('^'):
 	carriage_return();
 	erase_eol();		/* erase the prompt */
 #ifdef MAILCALL
 	setmail(TRUE);		/* force a mail check */
 #endif
 	return AS_ASK;
+    case 'B':				/* back up one line */
     case 'b': case Ctl('b'):		/* back up a page */
 	if (art <= lastart) {
 	    ART_LINE target;
@@ -1068,7 +1093,15 @@ run_the_selector:
 	    reread = TRUE;
 	    clear();
 	    do_fseek = TRUE;
-	    target = topline - (LINES - 2);
+	    if (*buf == 'B')
+		target = topline - 1;
+	    else {
+		target = topline - (LINES - 2);
+		if (marking && (marking_areas & BACKPAGE_MARKING)) {
+		    extern ART_LINE highlight;
+		    highlight = topline;
+		}
+	    }
 	    artline = topline;
 	    if (artline >= 0) do {
 		artline--;
@@ -1102,6 +1135,10 @@ run_the_selector:
 	forcegrow = TRUE;		/* recalculate lastart */
 	return AS_ASK;
     }
+    case Ctl('f'): {			/* forward? */
+	forward();
+	return AS_ASK;
+    }
     case '|':
     case 'w': case 'W':
     case 's': case 'S':		/* save command */
@@ -1170,6 +1207,12 @@ run_the_selector:
 		printf("\nDeselected all articles in this subject.\n");
 	    }
 	    return AS_ASK;
+#ifdef CHARSUBST
+	case 'C':
+	    if (!*(++charsubst))
+		charsubst = charsets;
+	    goto refresh_screen;
+#endif
 	case 'a':  case 's':  case 't':  case 'T':
 	    *buf = buf[1];
 	    goto run_the_selector;
@@ -1201,7 +1244,7 @@ bool_int force;
 	else
 	    mailcall = getval("MAILCALL","(Mail) ");
     }
-    mailcount %= 10;			/* check every 10 articles */
+    mailcount %= 5;			/* check every 5 articles */
 }
 #endif
 
@@ -1240,13 +1283,12 @@ ask_catchup()
 reask_catchup:
 #ifdef VERBOSE
     IF(verbose)
-	in_char("Do you really want to mark everything as read? [yn] ", 'C');
+	in_char("Do you really want to mark everything as read?",'C',"yn");
     ELSE
 #endif
 #ifdef TERSE
-	in_char("Really? [ynh] ", 'C');
+	in_char("Really?",'C',"ynh");
 #endif
-    setdef(buf,"y");
 #ifdef VERIFY
     printcmd();
 #endif
@@ -1297,8 +1339,10 @@ u to mark all and unsubscribe.\n\n\
 	    yankback();
 	putchar('\n') FLUSH;
     }
-    if (ch == 'u')
+    if (ch == 'u') {
 	rcchar[ng] = NEGCHAR;
+	printf("(If you meant to hit 'y' instead of 'u', press '-'.)\n");
+    }
     return ch;
 }
 
@@ -1316,10 +1360,9 @@ char_int ch;
 
     if (!use_one_line)
 	putchar('\n') FLUSH;
-    sprintf(cmd_buf,"Memorize %s command: [+.j,cC]", mode_string);
+    sprintf(cmd_buf,"Memorize %s command:", mode_string);
 reask_memorize:
-    in_char(cmd_buf, 'm');
-    setdef(buf,"+");
+    in_char(cmd_buf, 'm', "+.j,cC");
 #ifdef VERIFY
     printcmd();
 #endif
