@@ -75,9 +75,11 @@ access_ng()
 # ifdef TERSE
 		printf("\nNo spool for %s!\n",ngname) FLUSH;
 # endif
+#if 0
 # ifdef CATCHUP
 	    catch_up(ng);
 # endif
+#endif
 	} else {
 # ifdef VERBOSE
 	    IF(verbose)
@@ -106,6 +108,7 @@ access_ng()
 #endif /* !USE_NNTP */
 
     dmcount = 0;
+    in_ng = TRUE;			/* tell the world we are here */
 
     build_cache();
     return TRUE;
@@ -139,7 +142,9 @@ ART_NUM newlast;
 #endif
 	if (has_normal_kills) {
 	    bool forcelast_save = forcelast;
+	    ARTICLE *artp_save = artp;
 	    kill_unwanted(tmpfirst,buf,TRUE);
+	    artp = artp_save;
 	    forcelast = forcelast_save;
 	}
 #endif
@@ -239,17 +244,17 @@ register NG_NUM num;
 	    rcchar[num] = NEGCHAR;
 	return TR_BOGUS;
     }
-	
+
 #ifdef DEBUG
     if (debug & DEB_SOFT_POINTERS) {
 	printf("Should be %ld\n",(long)softptr[num]) FLUSH;
     }
 #endif
-#ifndef ANCIENT_NEWS
-    sscanf(tmpbuf+len+1, "%ld %ld %c", &last, &first, &ch);
-#else
+#ifdef ANCIENT_NEWS
     sscanf(tmpbuf+len+1, "%ld %c", &last, &ch);
     first = 1;
+#else
+    sscanf(tmpbuf+len+1, "%ld %ld %c", &last, &first, &ch);
 #endif
     if (!abs1st[num])
 	abs1st[num] = (ART_NUM)first;
