@@ -9,12 +9,12 @@
 
 bool	nntp_group _((char*,NG_NUM));
 bool	nntp_stat _((ART_NUM));
+ART_NUM	nntp_stat_id _((char*));
 bool	nntp_header _((ART_NUM));
 FILE	*nntp_body _((ART_NUM));
 time_t	nntp_time _((void));
 bool	nntp_newgroups _((time_t));
 bool	nntp_listgroup _((void));
-char	*nntp_get_a_line _((char*, int));
 char	*nntp_artname _((void));
 char	nntp_handle_timeout _((bool_int));
 void	nntp_cleanup _((void));
