@@ -9,11 +9,15 @@
 
 int	server_init _((char*));
 
-void	nntp_connect _((void));
+int	nntp_connect _((void));
 void	nntp_command _((char*));
 char	nntp_check _((bool_int));
 int	nntp_gets _((char*, int));
-void	nntp_close _((void));
+void	nntp_close _((bool_int));
+
+#define NNTP_LIST_END(s)  ((s)[0]=='.' && ((s)[1]=='\0' || (s)[1]=='\r'))
+
+#define nntp_get_a_line(buf,len,realloc) get_a_line(buf,len,realloc,ser_rd_fp)
 
 /* RFC 977 defines these, so don't change them */
 
