@@ -55,21 +55,14 @@ rcstuff_init()
     register bool foundany = FALSE;
     char *some_buf;
     long length;
-#ifdef USE_NNTP
-    char *cp;
-#endif
 
     /* make filenames */
 
 #ifdef USE_NNTP
-    if (cp = getenv("NEWSRC"))
-	rcname = savestr(filexp(cp));
-    else
-	rcname = savestr(filexp(RCNAME));
-#else
-    rcname = savestr(filexp(RCNAME));
+    if (getenv("NEWSRC"))
+	printf("NEWSRC environment variable ignored -- use DOTDIR to set the directory.\n") FLUSH;
 #endif
-
+    rcname = savestr(filexp(RCNAME));
     rctname = savestr(filexp(RCTNAME));
     rcbname = savestr(filexp(RCBNAME));
     softname = savestr(filexp(SOFTNAME));
@@ -90,7 +83,7 @@ rcstuff_init()
     /* read in the .newsrc file */
 
     for (nextrcline = 0;
-	(some_buf = get_a_line(buf,LBUFLEN,rcfp)) != Nullch;
+	(some_buf = get_a_line(buf,LBUFLEN,FALSE,rcfp)) != Nullch;
 	nextrcline++)			/* for each line in .newsrc */
     {
 	char tmpbuf[10];
@@ -237,7 +230,7 @@ NG_NUM ngnum;
     if ((rcfp = fopen(rcbname, "r")) != Nullfp) {
 	int length = rcnums[ngnum] - 1;
 
-	while ((some_buf = get_a_line(buf,LBUFLEN,rcfp)) != Nullch) {
+	while ((some_buf = get_a_line(buf,LBUFLEN,FALSE,rcfp)) != Nullch) {
 	    if (len_last_line_got <= 0)
 		continue;
 	    some_buf[len_last_line_got-1] = '\0';	/* wipe out newline */
@@ -305,6 +298,7 @@ int flags;
 	printf("\nBad newsgroup name.\n") FLUSH;
       check_fuzzy_match:
 	if (fuzzyGet && (flags & GNG_FUZZY)) {
+	    flags &= ~GNG_FUZZY;
 	    if (find_close_match())
 		what = ngname;
 	    else
@@ -317,13 +311,7 @@ int flags;
     if (ng == nextrcline) {		/* not in .newsrc? */
 	if (ng >= maxrcline)		/* check for overflow */
 	    grow_rc_arrays(maxrcline + 25);
-#ifdef USE_NNTP
-	softptr[ng] = 0;
-	if (!nntp_group(ngname,ng))
-#else /* !USE_NNTP */
-	if ((softptr[ng] = findact(buf,ngname,strlen(ngname),0L)) < 0)
-#endif /* !USE_NNTP */
-	{
+	if ((softptr[ng] = findact(buf,ngname,strlen(ngname),0L)) < 0) {
 	    dingaling();
 #ifdef VERBOSE
 	    IF(verbose)
@@ -357,15 +345,14 @@ int flags;
 	} else {
 #ifdef VERBOSE
 	IF(verbose)
-	    sprintf(promptbuf,"\nNewsgroup %s not in .newsrc -- subscribe? [ynYN] ",ngname);
+	    sprintf(promptbuf,"\nNewsgroup %s not in .newsrc -- subscribe?",ngname);
 	ELSE
 #endif
 #ifdef TERSE
-	    sprintf(promptbuf,"\nSubscribe %s? [ynYN] ",ngname);
+	    sprintf(promptbuf,"\nSubscribe %s?",ngname);
 #endif
 reask_add:
-	in_char(promptbuf,'A');
-	setdef(buf,"y");
+	in_char(promptbuf,'A',"ynYN");
 #ifdef VERIFY
 	printcmd();
 #endif
@@ -430,17 +417,16 @@ Type N to leave all remaining new groups unsubscribed.\n", ngname)
 #ifdef VERBOSE
 	IF(verbose)
 	    sprintf(promptbuf,
-"\nNewsgroup %s is unsubscribed -- resubscribe? [yn] ",ngname)
+"\nNewsgroup %s is unsubscribed -- resubscribe?",ngname)
   FLUSH;
 	ELSE
 #endif
 #ifdef TERSE
-	    sprintf(promptbuf,"\nResubscribe %s? [yn] ",ngname)
+	    sprintf(promptbuf,"\nResubscribe %s?",ngname)
 	      FLUSH;
 #endif
 reask_unsub:
-	in_char(promptbuf,'R');
-	setdef(buf,"y");
+	in_char(promptbuf,'R',"yn");
 #ifdef VERIFY
 	printcmd();
 #endif
@@ -806,8 +792,7 @@ cleanup_rc()
 	}
 #ifdef DELBOGUS
 reask_bogus:
-	in_char("Delete bogus newsgroups? [ny] ", 'D');
-	setdef(buf,"n");
+	in_char("Delete bogus newsgroups?", 'D', "ny");
 #ifdef VERIFY
 	printcmd();
 #endif
@@ -931,6 +916,17 @@ newsrc_check()
 	putchar('\n') FLUSH;
     }
     else {
+	/* File exists; if zero length and backup isn't, complain */
+	if (fstat(fileno(rcfp),&filestat) < 0) {
+	    perror(rcname);
+	    finalize(1);
+	}
+	if (filestat.st_size == 0
+	 && stat(rcbname,&filestat) >= 0 && filestat.st_size > 0) {
+	    printf("Warning: %s is zero length but %s is not.\n",rcname,rcbname);
+	    printf("Either recover your newsrc or else remove the backup copy.\n");
+	    finalize(1);
+	}
 	UNLINK(rcbname);		/* unlink backup file name */
 	safelink(rcname,rcbname);	/* and backup current name */
     }
