@@ -26,6 +26,7 @@
 #include "artio.h"
 #include "final.h"
 #include "decode.h"
+#include "charsubst.h"
 #include "INTERN.h"
 #include "respond.h"
 
@@ -54,7 +55,7 @@ save_article()
     if ((use_pref = isupper(cmd)) != 0)
 	cmd = tolower(cmd);
     parseheader(art);
-#ifdef METAMAIL
+#ifdef MIMESTORE
     savefrom = (!mime_article && (cmd == 'w' || cmd == 'e'))
 #else
     savefrom = (cmd == 'w' || cmd == 'e')
@@ -166,7 +167,7 @@ Saving null articles is not very productive!  :-)\n\
 	    while(fgets(art_buf,LBUFLEN,artfp) != Nullch) {
 		if (*art_buf <= ' ')
 		    continue;	/* Ignore empty or initially-whitespace lines */
-#ifdef METAMAIL
+#ifdef MIMESTORE
 		if (mime_article) {
 		    char oldmode = mode;
 		    if (!custom_extract) {
@@ -342,8 +343,7 @@ Saving null articles is not very productive!  :-)\n\
 	    s = buf;
 	}
 	for (iter = 0;
-	    (there = stat(s,&filestat) >= 0) &&
-	    (filestat.st_mode & S_IFDIR);
+	    (there = stat(s,&filestat) >= 0) && S_ISDIR(filestat.st_mode);
 	    iter++) {			/* is it a directory? */
 
 	    c = (s+strlen(s));
@@ -371,12 +371,10 @@ Saving null articles is not very productive!  :-)\n\
 		char *dflt = (instr(savename,"%a", TRUE) ? "nyq" : "ynq");
 		
 		sprintf(cmd_buf,
-		"\nFile %s doesn't exist--\n	use mailbox format? [%s] ",
-		  s,dflt);
+		"\nFile %s doesn't exist--\n	use mailbox format?",s);
 	      reask_save:
-		in_char(cmd_buf, 'M');
+		in_char(cmd_buf, 'M', dflt);
 		putchar('\n') FLUSH;
-		setdef(buf,dflt);
 #ifdef VERIFY
 		printcmd();
 #endif
@@ -415,7 +413,7 @@ q to abort.\n\
 		}
 	    }
 	}
-	else if (filestat.st_mode & S_IFCHR)
+	else if (S_ISCHR(filestat.st_mode))
 	    mailbox = FALSE;
 	else {
 	    int tmpfd;
@@ -483,7 +481,7 @@ cancel_article()
 #ifdef VERBOSE
 	IF(verbose)
 	    fputs("\n\
-Cancelling null articles is your idea of fun?  :-)\n\
+Canceling null articles is your idea of fun?  :-)\n\
 ",stdout) FLUSH;
 	ELSE
 #endif
@@ -614,7 +612,10 @@ void
 reply()
 {
     bool incl_body = (*buf == 'R');
-    char *maildoer = savestr(filexp(getval("MAILPOSTER",MAILPOSTER)));
+#ifdef CHARSUBST
+    char hbuf[4*LBUFLEN];
+#endif
+    char *maildoer = savestr(getval("MAILPOSTER",MAILPOSTER));
 
     artopen(art);
     tmpfp = fopen(headname,"w");	/* open header file */
@@ -640,17 +641,74 @@ reply()
 	parseheader(art);
 	fseek(artfp,(long)htype[PAST_HEADER].ht_minpos,0);
 	while (fgets(buf,LBUFLEN,artfp) != Nullch) {
+#ifdef CHARSUBST
+	    strcharsubst(buf,hbuf);
+	    fprintf(tmpfp,"%s%s",indstr,hbuf);
+#else
 	    fprintf(tmpfp,"%s%s",indstr,buf);
+#endif
 	}
 	fprintf(tmpfp,"\n");
     }
     fclose(tmpfp);
-    interp(cmd_buf, (sizeof cmd_buf), maildoer);
+    safecpy(cmd_buf,filexp(maildoer),sizeof cmd_buf);
     invoke(cmd_buf,origdir);
     UNLINK(headname);		/* kill the header file */
 no_reply:
     free(maildoer);
 }
+  
+void
+forward()
+{
+#ifdef CHARSUBST
+    char hbuf[4*LBUFLEN];
+#endif
+    char *maildoer = savestr(getval("FORWARDPOSTER",FORWARDPOSTER));
+
+    artopen(art);
+    tmpfp = fopen(headname,"w");	/* open header file */
+    if (tmpfp == Nullfp) {
+	printf(cantcreate,headname) FLUSH;
+	goto no_forward;
+    }
+    interp(buf, (sizeof buf), getval("FORWARDHEADER",FORWARDHEADER));
+    fputs(buf,tmpfp);
+    if (!instr(maildoer,"%h",TRUE))
+#ifdef VERBOSE
+	IF(verbose)
+	    printf("\n%s\n(Above lines saved in file %s)\n",buf,headname)
+	      FLUSH;
+	ELSE
+#endif
+#ifdef TERSE
+	    printf("\n%s\n(Header in %s)\n",buf,headname) FLUSH;
+#endif
+    if (artfp != Nullfp) {
+	interp(buf, (sizeof buf), getval("FORWARDMSG",FORWARDMSG));
+	if (*buf)
+	    fprintf(tmpfp,"%s\n",buf);
+	parseheader(art);
+	fseek(artfp,0L,0);
+	while (fgets(buf,LBUFLEN,artfp) != Nullch) {
+#ifdef CHARSUBST
+	    strcharsubst(buf,hbuf);
+	    fprintf(tmpfp,"%s",hbuf);
+#else
+	    fprintf(tmpfp,"%s",buf);
+#endif
+	}
+	interp(buf, (sizeof buf), getval("FORWARDMSGEND",FORWARDMSGEND));
+	if (*buf)
+	    fprintf(tmpfp,"%s\n",buf);
+    }
+    fclose(tmpfp);
+    safecpy(cmd_buf,filexp(maildoer),sizeof cmd_buf);
+    invoke(cmd_buf,origdir);
+    UNLINK(headname);		/* kill the header file */
+no_forward:
+    free(maildoer);
+}
 
 void
 followup()
@@ -689,7 +747,12 @@ trim the quoted article down as much as possible.)\n\
 	parseheader(art);
 	fseek(artfp,(long)htype[PAST_HEADER].ht_minpos,0);
 	while (fgets(buf,LBUFLEN,artfp) != Nullch) {
+#ifdef CHARSUBST
+	    strcharsubst(buf,hbuf);
+	    fprintf(tmpfp,"%s%s",indstr,hbuf);
+#else
 	    fprintf(tmpfp,"%s%s",indstr,buf);
+#endif
 	}
 	fprintf(tmpfp,"\n");
     }
@@ -713,13 +776,17 @@ char *cmd,*dir;
     mode = 'x';
 #ifdef VERBOSE
     IF(verbose)
-	printf("\n(leaving cbreak mode; cwd=%s)\nInvoking command: %s\n\n",
-	    dir,cmd) FLUSH;
+	printf("\n(leaving cbreak mode; cwd=%s)",dir);
     ELSE
 #endif
 #ifdef TERSE
-	printf("\n(-cbreak; cwd=%s)\nInvoking: %s\n\n",dir,cmd) FLUSH;
+	printf("\n(-cbreak; cwd=%s)",dir);
 #endif
+#ifdef DEBUG
+    if (debug)
+	printf("\nInvoking command: %s",cmd);
+#endif
+    printf("\n\n") FLUSH;
     resetty();			/* make terminal well-behaved */
     doshell(sh,cmd);		/* do the command */
     noecho();			/* set no echo */
