@@ -343,7 +343,6 @@ read_roots()
 	    union { ARTICLE *ap; int num; } uni;
 	    if (i)
 		subj_ptr[0]->thread_link = subj_ptr[1];
-	    subj_ptr[0]->flags &= ~SF_THREAD;
 	    uni.num = p_root.articles;
 	    (*subj_ptr++)->thread = uni.ap;
 	}
@@ -586,7 +585,6 @@ tweak_data()
     for (sp = first_subject; sp; sp = sp->next) {
 	uni.ap = sp->thread;
 	sp->thread = article_array[uni.num];
-	sp->thread->subj->flags |= SF_THREAD;
     }
     art_ptr = article_array;
     for (count = total.article; count--; ) {
@@ -599,6 +597,11 @@ tweak_data()
 	    uni.ap = ap->sibling;
 	    ap->sibling = article_array[uni.num-1];
 	}
+    }
+
+    art_ptr = article_array;
+    for (count = total.article; count--; ) {
+	ap = *art_ptr++;
 	if (!(ap->flags & AF_MISSING))
 	    cache_article(ap);
     }
