@@ -39,7 +39,7 @@ ov_init()
     if (*ser_line == NNTP_CLASS_OK) {
 	do {
 	    nntp_gets(ser_line, sizeof ser_line);
-	} while (NNTP_LIST_END(ser_line));
+	} while (!NNTP_LIST_END(ser_line));
     }
 #endif
     return TRUE;
@@ -52,9 +52,7 @@ bool ov_files_have_xrefs = TRUE;	/* set once per session */
 bool ov_files_have_xrefs = FALSE;	/* set once per session */
 # endif
 #endif
-#ifdef USE_XOVER
-ART_NUM ov_next_art;
-#else
+#ifndef USE_XOVER
 FILE *ov_in;
 #endif
 
@@ -76,12 +74,14 @@ bool_int cheating;
 
     setspin(cheating? SPIN_BACKGROUND : SPIN_FOREGROUND);
 beginning:
-    if (ov_opened)
-	first = ov_next_art;
-    if (last < first)
+    for (ap = article_ptr(first); first <= last && (ap->flags & AF_CACHED); )
+	first++, ap++;
+    if (first > last)
 	goto exit;
     if (last - first > OV_CHUNK_SIZE + OV_CHUNK_SIZE/2 - 1)
 	last = first + OV_CHUNK_SIZE - 1;
+    for (ap = article_ptr(last); last > first && (ap->flags & AF_CACHED); )
+	last--, ap--;
     sprintf(ser_line, "XOVER %ld-%ld", (long)first, (long)last);
     nntp_command(ser_line);
     if (nntp_check(FALSE) != NNTP_CLASS_OK) {
@@ -93,7 +93,6 @@ beginning:
 	if (!ov_opened)
 	    printf("\nGetting overview file."), fflush(stdout);
 #endif
-    ov_next_art = last+1;
 
 #else /* !USE_XOVER */
 
@@ -113,11 +112,12 @@ beginning:
     artnum = first-1;
     for (;;) {
 #ifdef USE_XOVER
-	line = nntp_get_a_line(last_buf, last_buflen);
-	if (NNTP_LIST_END(line))
+	line = nntp_get_a_line(last_buf, last_buflen, last_buf != buf);
+	if (!line || NNTP_LIST_END(line))
 	    break;
 #else
-	if (!(line = get_a_line(last_buf, last_buflen, ov_in)))
+	line = get_a_line(last_buf, last_buflen, last_buf != buf, ov_in);
+	if (!line)
 	    break;
 #endif
 	last_buf = line;
@@ -126,12 +126,14 @@ beginning:
 	spin(100);
 	if (artnum < first)
 	    continue;
-#ifndef USE_XOVER
 	if (artnum > last) {
 	    artnum = last;
+#ifdef USE_XOVER
+	    continue;
+#else
 	    break;
-	}
 #endif
+	}
 	if ((ap = ov_parse(line, artnum)) != Nullart) {
 #ifndef OV_XREFS
 	    if (ov_files_have_xrefs) {
@@ -178,7 +180,12 @@ beginning:
 #endif /* !USE_XOVER */
     }
     cachemask = (ThreadedGroup? AF_THREADED : AF_CACHED);
-    for (an = first, ap = article_ptr(an); an <= artnum; an++, ap++) {
+#ifdef USE_XOVER
+    an = real_first;
+#else
+    an = first;
+#endif
+    for (ap = article_ptr(an); an <= artnum; an++, ap++) {
 	if (!(ap->flags & cachemask)) {
 #ifdef USE_NNTP
 	    onemissing(ap);
@@ -190,6 +197,7 @@ beginning:
     if (artnum > last_cached && artnum >= first)
 	last_cached = artnum;
 #ifdef USE_XOVER
+  exit:
     if (int_count || !success) {
 	int_count = 0;
 	success = FALSE;
@@ -198,17 +206,16 @@ beginning:
 	success = FALSE;
     } else if (last < real_last) {
 	if (!cheating || !input_pending()) {
+	    first = last+1;
 	    last = real_last;
 	    goto beginning;
 	}
 	success = FALSE;
-    } else
-	ov_next_art = absfirst;
+    }
     if (success && real_first <= first_cached) {
 	first_cached = real_first;
 	cached_all_in_range = TRUE;
     }
-  exit:
 #else
     if (success && first <= first_cached) {
 	first_cached = first;
@@ -237,10 +244,15 @@ ART_NUM artnum;
 	return Nullart;
 
     if (len_last_line_got > 0 && line[len_last_line_got-1] == '\n')
-	line[len_last_line_got-1] = '\0';
+#ifdef USE_NNTP
+	if (len_last_line_got > 1 && line[len_last_line_got-2] == '\r')
+	    line[len_last_line_got-2] = '\0';
+	else
+#endif
+	    line[len_last_line_got-1] = '\0';
 
     cp = line;
-    
+
     for (nf = 0; ; nf++) {
 	fields[nf] = cp;
 	if (nf == OV_OTHERS)
