@@ -36,6 +36,93 @@ static bool empty_ok;
 static bool disp_status_line;
 static bool clean_screen;
 
+/* CAA hacks: xterm mouse support */
+static char *sel_mouse_save;
+
+void
+sel_go_line(line)
+int line;
+{
+    int i;
+
+    for (i = 0; i < sel_item_cnt; i++) {
+	if (sel_items[i].line > line)
+	    break;
+    }
+    if (i > 0)
+	i--;
+    sel_item_index = i;
+}
+
+void
+sel_do_mouse(button,x,y)
+int button;	/* 0: button1  1: button2  2: button3  3: release */
+int x,y;
+{
+    switch (button) {
+    case 0:
+    case 1:
+	if (!y)
+	    sel_mouse_save = "<";
+	else if (y >= sel_last_line)
+	    sel_mouse_save = (button == 0)? " " : ">";
+	else {
+	    sel_go_line(y);
+	    if (button == 0)
+		sel_mouse_save = ".";
+	}
+	break;
+    case 2:
+	/* move forward or backwards a page:
+	 *   if cursor in top half: backwards
+	 *   if cursor in bottom half: forwards
+	 */
+	if (y<(LINES/2))
+	    sel_mouse_save = "<";
+	else
+	    sel_mouse_save = ">";
+	break;
+    case 3:
+	/* do range stuff here later? */
+	break;
+    }
+}
+
+void
+sel_mouse()
+{
+    int x,y;
+    int button;
+
+    sel_mouse_save = Nullch;
+    read_tty(buf,1);
+    button = (int)(buf[0]);
+    read_tty(buf,1);
+    x = (int)(buf[0])-33;
+    read_tty(buf,1);
+    y = (int)(buf[0])-33;
+    sel_do_mouse(button&3,x,y);
+
+    /* get the button-up event */
+    while (1) {
+	getcmd(buf);
+	if (*buf == 3)	/* got the button-up */
+	    break;
+	/* otherwise just eat any other keystrokes */
+    }
+
+    /* interpret the button-up event */
+    read_tty(buf,1);
+    button = (int)(buf[0]);
+    read_tty(buf,1);
+    x = (int)(buf[0])-33;
+    read_tty(buf,1);
+    y = (int)(buf[0])-33;
+    sel_do_mouse(button&3,x,y);
+    if (sel_mouse_save)
+	pushstring(sel_mouse_save,0);
+}
+
 /* Display a menu of threads/subjects/articles for the user to choose from.
 ** If "cmd" is '+' we display all the unread items and allow the user to mark
 ** them as selected and perform various commands upon them.  If "cmd" is 'U'
@@ -60,6 +147,7 @@ char_int cmd;
     empty_ok = FALSE;
 
     set_sel_mode(cmd);
+    xmouse_on();
 
     if (!cache_range(sel_rereading? absfirst : firstart, lastart)) {
 	sel_ret = '+';
@@ -92,7 +180,7 @@ char_int cmd;
 	sel_ret = '+';
 	goto sel_exit;
     }
-    init_pages();
+    init_pages(FILL_LAST_PAGE);
     sel_item_index = 0;
     *promptbuf = '\0';
     disp_status_line = FALSE;
@@ -180,6 +268,8 @@ reinp_selector:
     spin_char = ' ';
 #ifdef CONDSUB
     getcmd(buf);
+    if (*buf == ' ')
+	setdef(buf, sel_at_end? &end_char : &page_char);
     ch = *buf;
 #else
     getcmd(cmd_buf);	/* If no conditionals, don't allow macros */ 
@@ -196,18 +286,17 @@ reinp_selector:
 	}
 	disp_status_line = FALSE;
     }
+/* CAA: hack for mouse support */
+    if (ch == Ctl('c')) {
+	sel_mouse();
+	goto position_selector;
+    }
     if (ch == '-') {
 	got_dash = 1;
 	if (!can_home)
 	    putchar('-'), fflush(stdout);
 	goto reinp_selector;
     }
-    if (ch == ' ') {
-	if (sel_at_end)
-	    ch = end_char;
-	else
-	    ch = page_char;
-    }
     in_select = index(sel_chars, ch);
     if (in_select) {
 	j = in_select - sel_chars;
@@ -422,7 +511,6 @@ sel_exit:
     else
 	srchahead = 0;
 #endif
-/*    selected_only = (selected_count || !article_count);*/
     selected_only = (selected_count != 0);
     if (sel_ret != '#')
 	count_subjects(sel_ret == '+'? CS_RESELECT : CS_UNSELECT);
@@ -433,6 +521,7 @@ sel_exit:
 	artp = curr_artp;
     } else
 	top_article();
+    xmouse_off();
     return sel_ret;
 }
 
@@ -506,7 +595,7 @@ char_int ch;
 	return DS_POS;
     case Ctl('l'):
 	return DS_DISPLAY;
-    case Ctl('f'):
+    case Ctl('^'):
 	erase_eol();		/* erase the prompt */
 #ifdef MAILCALL
 	setmail(TRUE);		/* force a mail check */
@@ -579,7 +668,7 @@ char_int ch;
 	count_subjects(CS_NORM);
 	sel_page_sp = Nullsubj;
 	sel_page_app = Null(ARTICLE**);
-	init_pages();
+	init_pages(PRESERVE_PAGE);
 	return DS_DISPLAY;
     case 'U':
 	sel_cleanup();
@@ -602,15 +691,14 @@ char_int ch;
 	}
 	count_subjects(CS_NORM);
 	sel_item_index = 0;
-	init_pages();
+	init_pages(FILL_LAST_PAGE);
 	return DS_DISPLAY;
     case 'S':
 	if (!sel_rereading)
 	    sel_cleanup();
 	erase_eol();		/* erase the prompt */
     reask_output:
-	in_char("Selector mode:  Threads, Subjects, Articles? [tsa] ", 'o');
-	setdef(buf,"t");
+	in_char("Selector mode:  Threads, Subjects, Articles?", 'o', "tsa");
 #ifdef VERIFY
 	printcmd();
 #endif
@@ -644,7 +732,7 @@ q does nothing.\n\n\
 	}
 	set_sel_mode(*buf);
 	count_subjects(CS_NORM);
-	init_pages();
+	init_pages(FILL_LAST_PAGE);
 	return DS_DISPLAY;
     case 'O':
 	if (!sel_rereading)
@@ -652,11 +740,10 @@ q does nothing.\n\n\
 	erase_eol();		/* erase the prompt */
     reask_sort:
 	if (sel_mode == SM_ARTICLE)
-	    in_char("Order by Date, Subject, Author, subject-date Groups? [dsagDSAG] ",
-		    'q');
+	    in_char("Order by Date, Subject, Author, subject-date Groups?",
+		    'q', "dsagDSAG");
 	else
-	    in_char("Order by Date, Subject, or Count? [dscDSC] ", 'q');
-	setdef(buf,"d");
+	    in_char("Order by Date, Subject, or Count?", 'q', "dscDSC");
 #ifdef VERIFY
 	printcmd();
 #endif
@@ -717,7 +804,7 @@ q does nothing.\n\n\
 	count_subjects(CS_NORM);
 	sel_page_sp = Nullsubj;
 	sel_page_app = Null(ARTICLE**);
-	init_pages();
+	init_pages(FILL_LAST_PAGE);
 	return DS_DISPLAY;
     case 'R':
 	if (!sel_rereading)
@@ -726,7 +813,7 @@ q does nothing.\n\n\
 	count_subjects(CS_NORM);
 	sel_page_sp = Nullsubj;
 	sel_page_app = Null(ARTICLE**);
-	init_pages();
+	init_pages(FILL_LAST_PAGE);
 	return DS_DISPLAY;
     case 'E':
 	if (!sel_rereading)
@@ -735,7 +822,7 @@ q does nothing.\n\n\
 	count_subjects(CS_NORM);
 	sel_page_sp = Nullsubj;
 	sel_page_app = Null(ARTICLE**);
-	init_pages();
+	init_pages(FILL_LAST_PAGE);
 	empty_ok = TRUE;
 	return DS_DISPLAY;
     case 'X':  case 'D':  case 'J':
@@ -777,7 +864,7 @@ q does nothing.\n\n\
 	    count_subjects(CS_UNSELECT);
 	    if (article_count
 	     && (ch == 'J' || (ch == 'D' && !selected_count))) {
-		init_pages();
+		init_pages(FILL_LAST_PAGE);
 		sel_item_index = 0;
 		return DS_DISPLAY;
 	    }
@@ -806,7 +893,7 @@ q does nothing.\n\n\
 	    register SUBJECT *sp = (SUBJECT*)sel_items[sel_item_index].ptr;
 	    if (sel_mode == SM_THREAD) {
 		while (!sp->misc)
-		    sp = sp->next;
+		    sp = sp->thread_link;
 	    }
 	    artp = sp->articles;
 	}
@@ -815,7 +902,7 @@ q does nothing.\n\n\
 	switch (ask_memorize(ch)) {
 	case 'j':  case ',':
 	    count_subjects(sel_rereading ? CS_NORM : CS_UNSELECT);
-	    init_pages();
+	    init_pages(PRESERVE_PAGE);
 	    sprintf(buf,"Kill memorized.");
 	    disp_status_line = TRUE;
 	    return DS_DISPLAY;
@@ -842,7 +929,9 @@ q does nothing.\n\n\
 	}
 	return DS_ASK;
     case Ctl('k'):
+	xmouse_off();
 	edit_kfile();
+	xmouse_on();
 	return DS_DISPLAY;
     case ':':  case '/':  case '&':  case '!':
 	erase_eol();		/* erase the prompt */
@@ -852,11 +941,13 @@ q does nothing.\n\n\
 	    goto extend_done;
 	}
 	if (ch == '&' || ch == '!') {
+	    xmouse_off();
 	    one_command = TRUE;
 	    perform(buf, FALSE);
 	    one_command = FALSE;
 	    putchar('\n') FLUSH;
 	    clean_screen = FALSE;
+	    xmouse_on();
 	} else {
 	    int sel_art_save = selected_count;
 
@@ -900,7 +991,7 @@ q does nothing.\n\n\
 	    }
 	    /* Recount, in case something has changed. */
 	    count_subjects(sel_rereading ? CS_NORM : CS_UNSELECT);
-	    init_pages();
+	    init_pages(PRESERVE_PAGE);
 	    sel_item_index = 0;
 
 	    sel_art_save -= selected_count;
@@ -948,7 +1039,7 @@ q does nothing.\n\n\
 	    if (ch != 'u' && article_count) {
 		sel_page_sp = Nullsubj;
 		sel_page_app = Null(ARTICLE**);
-		init_pages();
+		init_pages(FILL_LAST_PAGE);
 		return DS_DISPLAY;
 	    }
 	    sel_ret = 'Z';
