@@ -537,7 +537,7 @@ int mode;
     case SPIN_FOREGROUND:
     case SPIN_BACKGROUND:
 	if (!spin_level++) {
-	    if ((spin_art = openart) != 0)
+	    if ((spin_art = openart) != 0 && artfp)
 		spin_tell = ftell(artfp);
 	    spin_count = 1;	/* not 0 to prevent immediate spin display */
 	    spin_place = 1;	/* start with slash... */
@@ -558,7 +558,8 @@ int mode;
 	}
 	if (spin_art) {
 	    artopen(spin_art);
-	    fseek(artfp,spin_tell,0);	/* do not screw up the pager */
+	    if (artfp)
+		fseek(artfp,spin_tell,0);   /* do not screw up the pager */
 	    spin_art = 0;
 	}
 	break;
