@@ -17,6 +17,7 @@
 #include "artstate.h"
 #include "rcstuff.h"
 #include "ngdata.h"
+#include "final.h"
 #include "kfile.h"
 #include "head.h"
 #include "util.h"
@@ -200,8 +201,7 @@ inc_art(sel_flag, rereading)
 bool_int sel_flag, rereading;
 {
     register ARTICLE *ap = artp;
-    int subj_mask = (sel_mode == SM_THREAD? SF_THREAD : 0)
-		  | (rereading? 0 : SF_VISIT);
+    int subj_mask = (rereading? 0 : SF_VISIT);
 
     /* Use the explicit article-order if it exists */
     if (artptr_list) {
@@ -237,8 +237,7 @@ bool_int sel_flag, rereading;
 	if (ap)
 	    sp = ap->subj;
 	else
-	    for (sp = first_subject; sp && (sp->flags&subj_mask) != subj_mask;)
-		sp = sp->next;
+	    sp = next_subj(Nullsubj, subj_mask);
 	if (!sp)
 	    goto num_inc;
 	do {
@@ -247,11 +246,7 @@ bool_int sel_flag, rereading;
 	    else
 		ap = first_art(sp);
 	    while (!ap) {
-		if (sel_mode == SM_THREAD && sp->thread)
-		    sp = sp->thread->subj;
-		while ((sp = sp->next) != Nullsubj
-		    && (sp->flags & subj_mask) != subj_mask)
-		    ;
+		sp = next_subj(sp, subj_mask);
 		if (!sp)
 		    break;
 		ap = first_art(sp);
@@ -300,8 +295,7 @@ dec_art(sel_flag, rereading)
 bool_int sel_flag, rereading;
 {
     register ARTICLE *ap = artp;
-    int subj_mask = (sel_mode == SM_THREAD? SF_THREAD : 0)
-		  | (rereading? 0 : SF_VISIT);
+    int subj_mask = (rereading? 0 : SF_VISIT);
 
     /* Use the explicit article-order if it exists */
     if (artptr_list) {
@@ -331,8 +325,7 @@ bool_int sel_flag, rereading;
 	if (ap)
 	    sp = ap->subj;
 	else
-	    for (sp = last_subject; sp && (sp->flags&subj_mask) != subj_mask;)
-		sp = sp->prev;
+	    sp = prev_subj(Nullsubj, subj_mask);
 	if (!sp)
 	    goto num_dec;
 	do {
@@ -341,11 +334,7 @@ bool_int sel_flag, rereading;
 	    else
 		ap = last_art(sp);
 	    while (!ap) {
-		if (sel_mode == SM_THREAD && sp->thread)
-		    sp = sp->thread->subj;
-		while ((sp = sp->prev) != Nullsubj
-		    && (sp->flags & subj_mask) != subj_mask)
-		    ;
+		sp = prev_subj(sp, subj_mask);
 		if (!sp)
 		    break;
 		ap = last_art(sp);
@@ -529,6 +518,50 @@ prev_art_with_subj()
     return TRUE;
 }
 
+SUBJECT *
+next_subj(sp, subj_mask)
+register SUBJECT *sp;
+int subj_mask;
+{
+    if (!sp)
+	sp = first_subject;
+    else if (sel_mode == SM_THREAD) {
+	ARTICLE *ap = sp->thread;
+	do {
+	    sp = sp->next;
+	} while (sp && sp->thread == ap);
+    }
+    else
+	sp = sp->next;
+
+    while (sp && (sp->flags & subj_mask) != subj_mask) {
+	sp = sp->next;
+    }
+    return sp;
+}
+
+SUBJECT *
+prev_subj(sp, subj_mask)
+register SUBJECT *sp;
+int subj_mask;
+{
+    if (!sp)
+	sp = last_subject;
+    else if (sel_mode == SM_THREAD) {
+	ARTICLE *ap = sp->thread;
+	do {
+	    sp = sp->prev;
+	} while (sp && sp->thread == ap);
+    }
+    else
+	sp = sp->prev;
+
+    while (sp && (sp->flags & subj_mask) != subj_mask) {
+	sp = sp->prev;
+    }
+    return sp;
+}
+
 /* Select a single article.
 */
 void
@@ -567,10 +600,6 @@ int sel_flags;
 	if (!(ap->subj->flags & sel_mask))
 	    selected_subj_cnt++;
 	ap->subj->flags = (ap->subj->flags&~SF_DEL) | sel_mask | SF_VISIT;
-	if (sel_mode == SM_THREAD) {
-	    if ((ap = ap->subj->thread) != NULL)
-		ap->subj->flags |= SF_VISIT;
-	}
     }
     selected_only = (selected_only || selected_count != 0);
 }
@@ -613,10 +642,6 @@ int sel_flags;
 	    selected_subj_cnt++;
 	subj->flags = (subj->flags & ~SF_DEL)
 		    | sel_mask | SF_VISIT | SF_WASSELECTED;
-	if (sel_mode == SM_THREAD) {
-	    if ((ap = subj->thread) != NULL)
-		ap->subj->flags |= SF_VISIT;
-	}
 	selected_only = TRUE;
     } else
 	subj->flags |= SF_WASSELECTED;
@@ -686,10 +711,6 @@ int sel_flags;
 	if (!(subj->flags & sel_mask))
 	    selected_subj_cnt++;
 	subj->flags = (subj->flags & ~SF_DEL) | sel_mask | SF_VISIT;
-	if (sel_mode == SM_THREAD) {
-	    if ((ap = subj->thread) != NULL)
-		ap->subj->flags |= SF_VISIT;
-	}
 	selected_only = TRUE;
     }
 }
@@ -802,6 +823,7 @@ deselect_all()
 void
 kill_arts_subject(ap, kill_flags)
 register ARTICLE *ap;
+int kill_flags;
 {
     if (ap->subj && ap->subj->articles)
 	kill_subject(ap->subj, kill_flags);
@@ -839,6 +861,7 @@ int kill_flags;
 void
 kill_arts_thread(ap, kill_flags)
 register ARTICLE *ap;
+int kill_flags;
 {
     if (ap->subj && ap->subj->thread)
 	kill_thread(ap->subj->thread, kill_flags);
@@ -919,10 +942,6 @@ SUBJECT *subj;
     }
     if (!sel_rereading && selected_only && !(subj->flags & SF_SEL)) {
 	subj->flags |= SF_SEL | SF_VISIT | SF_WASSELECTED;
-	if (sel_mode == SM_THREAD) {
-	    if ((ap = subj->thread) != NULL)
-		ap->subj->flags |= SF_VISIT;
-	}
 	selected_subj_cnt++;
     }
     subj->flags &= ~(SF_DEL|SF_DELSEL);
@@ -973,10 +992,6 @@ register ARTICLE *ap;
     if (!(sp->flags & sel_mask))
 	selected_subj_cnt++;
     sp->flags = (sp->flags & ~SF_DEL) | SF_SEL | SF_VISIT;
-    if (sel_mode == SM_THREAD) {
-	if ((ap = sp->thread) != NULL)
-	    ap->subj->flags |= SF_VISIT;
-    }
     selected_only = (selected_only || selected_count != 0);
 }
 
@@ -1062,22 +1077,19 @@ SUBJECT *sp;
 ** only choose from threads with selected articles.
 */
 void
-next_subject()
+visit_next_thread()
 {
     register SUBJECT *sp;
-    register ARTICLE *ap;
-    int subj_mask = (sel_mode == SM_THREAD? (SF_THREAD|SF_VISIT) : SF_VISIT);
+    register ARTICLE *ap = artp;
 
-    sp = ((ap = artp) ? ap->subj->next : first_subject);
-    for (; sp; sp = sp->next) {
-	if ((sp->flags & subj_mask) == subj_mask) {
-	    if ((ap = subj_art(sp)) != Nullart) {
-		art = article_num(ap);
-		artp = ap;
-		return;
-	    }
-	    reread = FALSE;
+    sp = (ap? ap->subj : Nullsubj);
+    while ((sp = next_subj(sp, SF_VISIT)) != Nullsubj) {
+	if ((ap = subj_art(sp)) != Nullart) {
+	    art = article_num(ap);
+	    artp = ap;
+	    return;
 	}
+	reread = FALSE;
     }
     artp = Nullart;
     art = lastart+1;
@@ -1088,22 +1100,19 @@ next_subject()
 ** only choose from threads with selected articles.
 */
 void
-prev_subject()
+visit_prev_thread()
 {
     register SUBJECT *sp;
-    register ARTICLE *ap;
-    int subj_mask = (sel_mode == SM_THREAD? (SF_THREAD|SF_VISIT) : SF_VISIT);
+    register ARTICLE *ap = artp;
 
-    sp = ((ap = artp) ? ap->subj->prev : last_subject);
-    for (; sp; sp = sp->prev) {
-	if ((sp->flags & subj_mask) == subj_mask) {
-	    if ((ap = subj_art(sp)) != Nullart) {
-		art = article_num(ap);
-		artp = ap;
-		return;
-	    }
-	    reread = FALSE;
+    sp = (ap? ap->subj : Nullsubj);
+    while ((sp = prev_subj(sp, SF_VISIT)) != Nullsubj) {
+	if ((ap = subj_art(sp)) != Nullart) {
+	    art = article_num(ap);
+	    artp = ap;
+	    return;
 	}
+	reread = FALSE;
     }
     artp = Nullart;
     art = lastart+1;
@@ -1337,10 +1346,6 @@ int cmode;
 	}
 	if (count && (!selected_only || (sp->flags & sel_mask))) {
 	    sp->flags |= SF_VISIT;
-	    if (sel_mode == SM_THREAD) {
-		if ((ap = sp->thread) != NULL)
-		    ap->subj->flags |= SF_VISIT;
-	    }
 	}
     }
     if (cmode != CS_RETAIN && !article_count && !selected_only) {
@@ -1478,7 +1483,7 @@ sort_subjects()
 	lp[0]->next = lp[1];
 	lp[1]->prev = lp[0];
 	if (sel_mode == SM_THREAD) {
-	    if (lp[0]->thread == lp[1]->thread)
+	    if (lp[0]->thread && lp[0]->thread == lp[1]->thread)
 		lp[0]->thread_link = lp[1];
 	    else {
 		lp[0]->thread_link = sp;
