@@ -188,7 +188,8 @@ char *tcbuf;		/* temp area for "uncompiled" termcap entry */
     CE = Tgetstr("ce");			/* clear to end of line string */
     TI = Tgetstr("ti");			/* initialize display */
     TE = Tgetstr("te");			/* reset display */
-    HO = Tgetstr("ho");			/* home cursor if no CM */
+    HO = Tgetstr("ho");			/* home cursor */
+    IL = Tgetstr("al");			/* insert (add) line */
     CM = Tgetstr("cm");			/* cursor motion */
     if (*CM || *HO)
 	can_home = TRUE;
@@ -302,23 +303,23 @@ char *tmpbuf;
      * want to redefine them.  (The tvi912c defines kl as ^H)
      */
     strcpy(lbuf,Tgetstr("ku"));		/* up */
-    /* '(' at article or pager, 'p' otherwise */
-    if (strlen(lbuf)>1)
-	set_macro(lbuf,"%(%m=[ap]?\\(:p)");
+    /* '(' at article or pager, '[' in thread sel, 'p' otherwise */
+    if ((int)strlen(lbuf) > 1)
+	set_macro(lbuf,"%(%m=[ap]?\\(:%(%m=t?[:p))");
 
     strcpy(lbuf,Tgetstr("kd"));		/* down */
-    /* ')' at article or pager, 'n' otherwise */
-    if (strlen(lbuf)>1)
-	set_macro(lbuf,"%(%m=[ap]?\\):n)");
+    /* ')' at article or pager, ']' in thread sel, 'n' otherwise */
+    if ((int)strlen(lbuf) > 1)
+	set_macro(lbuf,"%(%m=[ap]?\\):%(%m=t?]:n))");
 
     strcpy(lbuf,Tgetstr("kl"));		/* left */
     /* '[' at article or pager, '<' otherwise */
-    if (strlen(lbuf)>1)
+    if ((int)strlen(lbuf) > 1)
 	set_macro(lbuf,"%(%m=[ap]?\\[:<)");
 
     strcpy(lbuf,Tgetstr("kr"));		/* right */
     /* CR at newsgroups, ']' at article or pager, '>' otherwise */
-    if (strlen(lbuf)>1)
+    if ((int)strlen(lbuf) > 1)
 	set_macro(lbuf,"%(%m=n?^j:%(%m=[ap]?\\]:>))");
 }
 
@@ -328,6 +329,10 @@ char *tcbuf;
 {
     char tmpbuf[1024];
 
+    /* set up the Xterm mouse sequence */
+    sprintf(tmpbuf,"\033[M");	/* enter mouse escape sequence */
+    set_macro(tmpbuf,"\003");
+
     if (auto_arrow_macros)
 	arrow_macros(tmpbuf);
     if (use_threads)
@@ -661,10 +666,10 @@ re_read:
 	else if (*s == '\\') {
 	    quoteone = TRUE;
 	}
-#ifdef cray
-    } while (*s != '\r');		/* till a newline (not echoed) */
+#if defined(__bsdi__) || defined(cray)
+    } while (*s != '\r' && *s != '\n'); /* until CR or NL (not echoed) */
 #else
-    } while (*s != '\n');		/* till a newline (not echoed) */
+    } while (*s != '\n');		/* until a newline (not echoed) */
 #endif
     *s = '\0';				/* terminate the string nicely */
     if (donewline)
@@ -1036,15 +1041,16 @@ pause_getcmd()
 }
 
 void
-in_char(prompt, newmode)
+in_char(prompt, newmode, dflt)
 char *prompt;
 char_int newmode;
+char *dflt;
 {
     char oldmode = mode;
 
 reask_in_char:
     unflush_output();			/* disable any ^O in effect */
-    fputs(prompt,stdout);
+    printf("%s [%s] ", prompt, dflt);
     fflush(stdout);
     eat_typeahead();
     mode = newmode;
@@ -1053,6 +1059,7 @@ reask_in_char:
 	putchar('\n') FLUSH;		/* if return from stop signal */
 	goto reask_in_char;		/* give them a prompt again */
     }
+    setdef(buf,dflt);
     mode = oldmode;
 }
 
@@ -1332,18 +1339,40 @@ void
 termlib_init()
 {
 #ifdef USETITE
-    if (TI && *TI)
+    if (TI && *TI) {
 	tputs(TI,1,putchr);
+	fflush(stdout);
+    }
 #endif
-    return;
 }
 
 void
 termlib_reset()
 {
 #ifdef USETITE
-    if (TE && *TE)
+    if (TE && *TE) {
 	tputs(TE,1,putchr);
+	fflush(stdout);
+    }
 #endif
-    return;
+}
+
+void
+xmouse_on()
+{
+    if (use_xterm_mouse && !xmouse_status) {
+	printf("\033[?1001s");   /* save old highlight mouse tracking */
+	printf("\033[?1000h");   /* enable mouse tracking */
+	xmouse_status = TRUE;
+    }
+}
+
+void
+xmouse_off()
+{
+    if (use_xterm_mouse && xmouse_status) {
+	printf("\033[?10001");   /* disable mouse tracking */
+	printf("\033[?1001r");   /* restore old highlight mouse tracking */
+	xmouse_status = FALSE;
+    }
 }
