@@ -266,8 +266,17 @@ newsgroup use the g<newsgroup> command.\n\
 			    ng = current_ng;
 		    special = TRUE;	/* don't skip it if toread==0 */
 		    break;
-		case 'q': case 'Q': case 'x':	/* quit? */
-		    oh_for_the_good_old_days = (*buf == 'x');
+		case 'x':
+		    putchar('\n');
+		    in_char("Confirm: exit and abandon .newsrc changes?",
+			    'A', "yn");
+		    putchar('\n') FLUSH;
+		    if (*buf != 'y')
+			break;
+		    oh_for_the_good_old_days = TRUE;
+		    printf("\nMoving changed version to '.newnewsrc'.");
+		    /* FALL THROUGH */
+		case 'q': case 'Q':	/* quit? */
 		    putchar('\n') FLUSH;
 		    ng = nextrcline+1;	/* satisfy */
 		    retry = FALSE;	/*   loop conditions */
@@ -440,13 +449,13 @@ newsgroup use the g<newsgroup> command.\n\
 reask_abandon:
 #ifdef VERBOSE
 		    IF(verbose)
-			in_char("\nAbandon changes to current newsgroup? [yn] ", 'B');
+			in_char("\nAbandon changes to current newsgroup?",
+				'B', "yn");
 		    ELSE
 #endif
 #ifdef TERSE
-			in_char("\nAbandon? [ynh] ", 'B');
+			in_char("\nAbandon?", 'B', "ynh");
 #endif
-		    setdef(buf,"y");
 #ifdef VERIFY
 		    printcmd();
 #endif
